@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bbs_thread_ui.c,v 1.93 2004/01/20 13:49:21 fuyu Exp $
+ * $Id: bbs_thread_ui.c,v 1.94 2004/01/22 07:43:02 fuyu Exp $
  */
 
 #include "config.h"
@@ -1254,7 +1254,7 @@ link_mouse_press_cb(BBSThreadView *view, GdkEventButton *event,
       int res_num;
       if (!application->stick_bookmark)
 	{
-	  return FALSE;
+	  return event->button == 3;
 	}
 
       res_num = 0;
