@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: download_ui.c,v 1.8 2004/01/21 08:41:58 fuyu Exp $
+ * $Id: download_ui.c,v 1.9 2004/01/22 07:45:09 fuyu Exp $
  */
 
 #include "config.h"
@@ -204,7 +204,7 @@ download_dialog_response_cb(GtkWidget *widget, int response_id,
 	= (GtkEntry *)g_object_get_qdata(G_OBJECT(widget), filename_entry_id);
       const char *filename = gtk_entry_get_text(entry);
 
-      int fd = open(filename, O_CREAT | O_TRUNC | O_WRONLY);
+      int fd = open(filename, O_CREAT | O_TRUNC | O_WRONLY, 0x1a4);
       if (fd > 0)
 	{
 	  const char *url
@@ -606,9 +606,9 @@ download_image(WorkerThread *employee, gpointer args)
 	while (TRUE)
 	  {
 	    size_t length = buffer->length;
-	    size_t updated_len = length - offset;
-	    if (updated_len > 0)
+	    if (length > 0 && length > offset)
 	      {
+		size_t updated_len = length - offset;
 		OCHU_THREADS_ENTER();
 		gdk_pixbuf_loader_write(loader,
 					((const guchar *)buffer->buffer
