@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_ui_private.h,v 1.6 2004/01/15 09:33:30 fuyu Exp $
+ * $Id: ochusha_ui_private.h,v 1.9 2004/01/20 13:50:58 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_UI_PRIVATE_H_
@@ -54,6 +54,9 @@ extern volatile int last_line;
 extern pthread_t main_thread;
 #endif
 
+void ochu_threads_enter(void);
+void ochu_threads_leave(void);
+
 #define OCHU_THREADS_ENTER()						\
   do									\
     {									\
@@ -90,6 +93,9 @@ extern pthread_t main_thread;
       gdk_threads_leave();						\
     } while (0)
 
+#define OCHU_THREADS_ENTER_FUNC		ochu_threads_enter
+#define OCHU_THREADS_LEAVE_FUNC		ochu_threads_leave
+
 #define MAIN_THREAD_EVENT_CALLBACK()					\
   do									\
     {									\
@@ -140,6 +146,8 @@ extern pthread_t main_thread;
 
 #define OCHU_THREADS_ENTER()		gdk_threads_enter()
 #define OCHU_THREADS_LEAVE()		gdk_threads_leave()
+#define OCHU_THREADS_ENTER_FUNC		gdk_threads_enter
+#define OCHU_THREADS_LEAVE_FUNC		gdk_threads_leave
 #define CHECK_THREADS_ENTERED()
 #define MAIN_THREAD_EVENT_CALLBACK()
 #define ENTER_MAIN_EVENT_LOOP()
