@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: paned_notebook.c,v 1.19 2004/01/14 10:03:17 fuyu Exp $
+ * $Id: paned_notebook.c,v 1.21 2004/01/16 08:01:46 fuyu Exp $
  */
 
 #include "config.h"
@@ -1289,6 +1289,31 @@ paned_notebook_close_item_view(PanedNotebook *paned_notebook, gpointer item)
 }
 
 
+/*
+ * viewɽǤä餽ΥڡĤ롣
+ */
+void
+paned_notebook_close_view(PanedNotebook *paned_notebook, GtkWidget *view)
+{
+  ItemViewRelation *relation = find_item_view_relation(paned_notebook, view);
+  int page;
+
+  if (relation == NULL)
+    return;
+
+  page = mod_notebook_page_num(paned_notebook->item_contents,
+			       relation->item_view);
+  if (page == -1)
+    {
+      /* ɽƤϤΥåȤĤʤ */
+      fprintf(stderr, "Something wrong happen.\n");
+      return;
+    }
+
+  close_nth_page(paned_notebook, page);
+}
+
+
 void
 paned_notebook_close_current_item_view(PanedNotebook *paned_notebook)
 {
@@ -1430,8 +1455,6 @@ button_press_event_cb(ModNotebook *notebook, GdkEventButton *event,
 {
   g_return_val_if_fail(notebook == paned_notebook->item_contents, FALSE);
 
-  MAIN_THREAD_EVENT_CALLBACK();
-
   if (event->button == 2)
     {
       int page = mod_notebook_get_mouse_event_page_num(notebook,
@@ -1443,8 +1466,6 @@ button_press_event_cb(ModNotebook *notebook, GdkEventButton *event,
 	}
     }
 
-  ENTER_MAIN_EVENT_LOOP();
-
   return FALSE;
 }
 
@@ -1496,31 +1517,19 @@ menu_item_activate_cb(GtkMenuItem *menu_item, PanedNotebook *paned_notebook)
 static void
 close_item_view_button_cb(gpointer unused, PanedNotebook *paned_notebook)
 {
-  MAIN_THREAD_EVENT_CALLBACK();
-
   paned_notebook_close_current_item_view(paned_notebook);
-
-  ENTER_MAIN_EVENT_LOOP();
 }
 
 
 static void
 go_left_button_cb(gpointer unused, PanedNotebook *paned_notebook)
 {
-  MAIN_THREAD_EVENT_CALLBACK();
-
   mod_notebook_prev_page(paned_notebook->item_contents);
-
-  ENTER_MAIN_EVENT_LOOP();
 }
 
 
 static void
 go_right_button_cb(gpointer unused, PanedNotebook *paned_notebook)
 {
-  MAIN_THREAD_EVENT_CALLBACK();
-
   mod_notebook_next_page(paned_notebook->item_contents);
-
-  ENTER_MAIN_EVENT_LOOP();
 }
