@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: paned_notebook.h,v 1.6 2004/01/15 02:18:05 fuyu Exp $
+ * $Id: paned_notebook.h,v 1.7 2004/01/15 15:05:35 fuyu Exp $
  */
 
 #ifndef _PANED_NOTEBOOK_H_
@@ -182,6 +182,8 @@ void paned_notebook_open_item_view(PanedNotebook *paned_notebook,
 /* itemɽǤʤitem_view_requiredȯ */
 void paned_notebook_close_item_view(PanedNotebook *paned_notebook,
 				    gpointer item);
+void paned_notebook_close_view(PanedNotebook *paned_notebook,
+			       GtkWidget *widget);
 /* itemɽäitem_view_closedȯ */
 void paned_notebook_close_current_item_view(PanedNotebook *paned_notebook);
 
