@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: virtual_board.c,v 1.16 2004/01/18 04:50:59 fuyu Exp $
+ * $Id: virtual_board.c,v 1.17 2004/01/22 07:46:38 fuyu Exp $
  */
 
 #include "config.h"
@@ -553,8 +553,16 @@ virtual_board_refresh_threadlist(OchushaBulletinBoard *board,
   if ((args.mode == OCHUSHA_NETWORK_BROKER_CACHE_ONLY
        || info->use_static_threadlist)
       && virtual_board->recent_thread_list != NULL)
-    g_slist_foreach(virtual_board->recent_thread_list,
-		    (GFunc)each_thread_cb, callback_data);
+    {
+      if (start_parsing_cb != NULL)
+	(*start_parsing_cb)(callback_data);
+
+      g_slist_foreach(virtual_board->recent_thread_list,
+		      (GFunc)each_thread_cb, callback_data);
+
+      if (end_parsing_cb != NULL)
+	(*end_parsing_cb)(callback_data);
+    }
   else
     {
       GSList *previous_recent_list = virtual_board->recent_thread_list;
@@ -564,7 +572,7 @@ virtual_board_refresh_threadlist(OchushaBulletinBoard *board,
 	{
 	  virtual_board->recent_thread_list = args.recent_list;
 	  g_slist_free(previous_recent_list);
-	  /* ȤäĤget_threadlist_sourceޤǺѲǽ */
+	  /* ȤäĤget_threadlist_sourceޤǺѤ */
 	  g_object_set_qdata(G_OBJECT(buffer), cache_mode_id,
 			     (gpointer)OCHUSHA_NETWORK_BROKER_CACHE_ONLY);
 	}
