@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_async_buffer.c,v 1.26 2004/01/20 13:52:37 fuyu Exp $
+ * $Id: ochusha_async_buffer.c,v 1.27 2004/01/21 17:25:59 fuyu Exp $
  */
 
 #include "config.h"
@@ -126,6 +126,12 @@ static GSList *sync_object_list = NULL;
  */
 static GSList *ochusha_async_buffer_list = NULL;
 
+/*
+ * Active(number_of_active_users > 0)Ƥ
+ * OchushaAsyncBufferݻ
+ */
+static GSList *active_buffer_list = NULL;
+
 
 #define LOCK_GLOBAL_LOCK						\
   if (pthread_mutex_lock(&ochusha_async_buffer_global_lock) != 0)	\
@@ -280,7 +286,7 @@ force_awake_job(WorkerThread *employee, void *unused)
     {
       LOCK_GLOBAL_LOCK;
       to_be_signaled_buffer_list = NULL;
-      g_slist_foreach(ochusha_async_buffer_list,
+      g_slist_foreach(active_buffer_list,
 		      (GFunc)iterate_buffers_to_be_signaled, NULL);
       UNLOCK_GLOBAL_LOCK;
 
@@ -405,7 +411,14 @@ ochusha_async_buffer_finalize(GObject *object)
       sync_object_list = g_slist_append(sync_object_list, buffer->sync_object);
       buffer->sync_object = NULL;
     }
-  ochusha_async_buffer_list = g_slist_remove(ochusha_async_buffer_list, buffer);
+  ochusha_async_buffer_list = g_slist_remove(ochusha_async_buffer_list,
+					     buffer);
+  if (g_slist_find(active_buffer_list, buffer) != NULL)
+    {
+      active_buffer_list = g_slist_remove(active_buffer_list, buffer);
+      g_warning("OchushaAsyncBuffer(%p) is finalized in active state!.\n",
+		buffer);
+    }
   UNLOCK_GLOBAL_LOCK;
 
   if (G_OBJECT_CLASS(parent_class)->finalize)
@@ -452,13 +465,10 @@ ochusha_async_buffer_new(char *buffer, size_t length, DestructFunc *destructor)
   OchushaAsyncBuffer *buf
     = OCHUSHA_ASYNC_BUFFER(g_object_new(OCHUSHA_TYPE_ASYNC_BUFFER, NULL));
 
-  LOCK_GLOBAL_LOCK
-    {
-      sync_object = get_sync_object();
-      ochusha_async_buffer_list = g_slist_append(ochusha_async_buffer_list,
-						 buf);
-    }
-  UNLOCK_GLOBAL_LOCK
+  LOCK_GLOBAL_LOCK;
+  sync_object = get_sync_object();
+  ochusha_async_buffer_list = g_slist_append(ochusha_async_buffer_list, buf);
+  UNLOCK_GLOBAL_LOCK;
 
   if (buffer == NULL && destructor == NULL)
     {
@@ -495,13 +505,10 @@ ochusha_async_buffer_new_with_file(int fd)
   gzFile gzfile;
   int chunk_size;
 
-  LOCK_GLOBAL_LOCK
-    {
-      sync_object = get_sync_object();
-      ochusha_async_buffer_list = g_slist_append(ochusha_async_buffer_list,
-						 buf);
-    }
-  UNLOCK_GLOBAL_LOCK
+  LOCK_GLOBAL_LOCK;
+  sync_object = get_sync_object();
+  ochusha_async_buffer_list = g_slist_append(ochusha_async_buffer_list, buf);
+  UNLOCK_GLOBAL_LOCK;
 
   buf->fixed = FALSE;
 
@@ -576,13 +583,10 @@ ochusha_async_buffer_new_with_file_mmap(int fd)
     = OCHUSHA_ASYNC_BUFFER(g_object_new(OCHUSHA_TYPE_ASYNC_BUFFER, NULL));
   off_t len;
 
-  LOCK_GLOBAL_LOCK
-  {
-    sync_object = get_sync_object();
-    ochusha_async_buffer_list = g_slist_append(ochusha_async_buffer_list,
-					       buf);
-  }
-  UNLOCK_GLOBAL_LOCK
+  LOCK_GLOBAL_LOCK;
+  sync_object = get_sync_object();
+  ochusha_async_buffer_list = g_slist_append(ochusha_async_buffer_list, buf);
+  UNLOCK_GLOBAL_LOCK;
 
   len = lseek(fd, 0, SEEK_END);
   buffer = mmap(NULL, len, PROT_READ, MAP_NOCORE | MAP_PRIVATE, fd, 0);
@@ -677,6 +681,14 @@ ochusha_async_buffer_active_ref(OchushaAsyncBuffer *buffer)
       g_object_unref(G_OBJECT(buffer));	/* Ĵ٤Ƥʤ */
     }
 
+  if (buffer->number_of_active_users == 1)
+    {
+      LOCK_GLOBAL_LOCK;
+      g_assert(g_slist_find(active_buffer_list, buffer) == NULL);
+      active_buffer_list = g_slist_append(active_buffer_list, buffer);
+      UNLOCK_GLOBAL_LOCK;
+    }
+
   UNLOCK_BUFFER_LOCK(buffer);
 
   return result;
@@ -703,6 +715,14 @@ ochusha_async_buffer_active_unref(OchushaAsyncBuffer *buffer)
       UNLOCK_GLOBAL_LOCK;
     }
 
+  if (buffer->number_of_active_users == 0)
+    {
+      LOCK_GLOBAL_LOCK;
+      g_assert(g_slist_find(active_buffer_list, buffer) != NULL);
+      active_buffer_list = g_slist_remove(active_buffer_list, buffer);
+      UNLOCK_GLOBAL_LOCK;
+    }
+
   UNLOCK_BUFFER_LOCK(buffer);
 
   g_object_unref(G_OBJECT(buffer));	/* Ĵ٤Ƥʤ */
