@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bbs_thread.h,v 1.21 2004/01/15 02:12:14 fuyu Exp $
+ * $Id: ochusha_bbs_thread.h,v 1.22 2004/01/18 04:50:26 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_BBS_THREAD_H_
@@ -112,6 +112,10 @@ struct _OchushaBBSThreadClass
 			      EachResponseCallback *each_response_callback,
 			      BrokenResponseCallback *broken_response_callback,
 			      EndThreadCallback *end_thread_callback,
+			      StartParsingCallback *start_parsing_callback,
+			      BeforeWaitCallback *before_wait_callback,
+			      AfterWaitCallback *after_wait_callback,
+			      EndParsingCallback *end_parsing_callback,
 			      gpointer callback_data);
   time_t (*get_last_modified_utc)(OchushaBBSThread *thread);
   const char *(*get_url)(OchushaBBSThread *thread);
@@ -197,6 +201,10 @@ gboolean ochusha_bbs_thread_parse_responses(OchushaBBSThread *thread,
 					    EachResponseCallback *response_cb,
 					    BrokenResponseCallback *broken_cb,
 					    EndThreadCallback *end_cb,
+					    StartParsingCallback *start_parsing_callback,
+					    BeforeWaitCallback *before_wait_callback,
+					    AfterWaitCallback *after_wait_callback,
+					    EndParsingCallback *end_parsing_callback,
 					    gpointer callback_data);
 void ochusha_bbs_thread_remove_cache(OchushaBBSThread *thread,
 				     OchushaConfig *config);
