@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_board_2ch.c,v 1.35 2004/01/16 08:00:01 fuyu Exp $
+ * $Id: ochusha_board_2ch.c,v 1.37 2004/01/21 08:41:01 fuyu Exp $
  */
 
 #include "config.h"
@@ -96,6 +96,10 @@ static gboolean ochusha_board_2ch_refresh_threadlist(
 						OchushaBulletinBoard *board,
 						OchushaAsyncBuffer *buffer,
 						EachThreadCallback *cb,
+						StartParsingCallback *start_parsing_cb,
+						BeforeWaitCallback *before_wait_cb,
+						AfterWaitCallback *after_wait_cb,
+						EndParsingCallback *end_parsing_cb,
 						gpointer callback_data);
 
 static const char *ochusha_board_2ch_get_response_character_encoding(
@@ -379,6 +383,7 @@ ochusha_board_2ch_recover_threadlist(OchushaBulletinBoard *board,
 						     0, 1, TRUE,
 						     recover_title,
 						     NULL, NULL, NULL,
+						     NULL, NULL, NULL, NULL,
 						     NULL);
 		  OCHU_OBJECT_UNREF(dat_buffer);
 		  thread->number_of_responses_read = 1;
@@ -537,6 +542,7 @@ ochusha_board_2ch_lookup_kako_thread_by_url(OchushaBulletinBoard *board,
 						 dat_buffer,
 						 0, -1, FALSE,
 						 recover_title,
+						 NULL, NULL, NULL, NULL,
 						 NULL, NULL, NULL, NULL);
 	      ochusha_async_buffer_suspend(dat_buffer);
 	      ochusha_async_buffer_terminate(dat_buffer);
@@ -860,7 +866,8 @@ ochusha_board_2ch_get_threadlist_source(OchushaBulletinBoard *board,
 
   board_2ch = OCHUSHA_BOARD_2CH(board);
   return ochusha_network_broker_read_from_url(broker, buffer, url,
-					      board_2ch->last_modified, mode);
+					      board_2ch->last_modified, mode,
+					      FALSE);
 }
 
 
@@ -953,6 +960,10 @@ static gboolean
 refresh_threadlist_by_subject_txt(OchushaBulletinBoard *board,
 				  OchushaAsyncBuffer *buffer,
 				  EachThreadCallback *each_thread_cb,
+				  StartParsingCallback *start_parsing_cb,
+				  BeforeWaitCallback *before_wait_cb,
+				  AfterWaitCallback *after_wait_cb,
+				  EndParsingCallback *end_parsing_cb,
 				  gpointer callback_data)
 {
   char default_buffer[PATH_MAX];
@@ -999,6 +1010,9 @@ refresh_threadlist_by_subject_txt(OchushaBulletinBoard *board,
     unsigned int offset = 0;
     gboolean buffer_fixed = FALSE;
 
+    if (start_parsing_cb != NULL)
+      (*start_parsing_cb)(callback_data);
+
     while (TRUE)
       {
 	char *buffer_top = (char *)buffer->buffer;
@@ -1123,7 +1137,10 @@ refresh_threadlist_by_subject_txt(OchushaBulletinBoard *board,
 
 	if (interval > 0)
 	  {
-	    unsigned int old_len = buffer->length;
+	    size_t old_len = buffer->length;
+
+	    if (before_wait_cb != NULL)
+	      (*before_wait_cb)(callback_data);
 	    do
 	      {
 		if (!ochusha_async_buffer_wait(buffer))
@@ -1131,9 +1148,13 @@ refresh_threadlist_by_subject_txt(OchushaBulletinBoard *board,
 #if DEBUG_ASYNC_BUFFER_MOST
 		    fprintf(stderr, "buffer has been terminated.\n");
 #endif
+		    if (after_wait_cb != NULL)
+		      (*after_wait_cb)(callback_data);
 		    goto terminated;
 		  }
-	      } while (old_len > buffer->length && !buffer->fixed);
+	      } while (old_len >= buffer->length && !buffer->fixed);
+	    if (after_wait_cb != NULL)
+	      (*after_wait_cb)(callback_data);
 	  }
 	else
 	  {
@@ -1147,15 +1168,23 @@ refresh_threadlist_by_subject_txt(OchushaBulletinBoard *board,
 	    g_object_set_qdata(G_OBJECT(buffer), idle_checker_id, buffer);
 	    g_idle_add_full(G_PRIORITY_HIGH_IDLE + 30,
 			    advance_parsing, buffer, NULL);
+	    if (before_wait_cb != NULL)
+	      (*before_wait_cb)(callback_data);
 	    do
 	      {
 #if 0
 		count++;
 #endif
 		if (!ochusha_async_buffer_wait(buffer))
-		  goto terminated;
+		  {
+		    if (after_wait_cb != NULL)
+		      (*after_wait_cb)(callback_data);
+		    goto terminated;
+		  }
 	      } while (g_object_get_qdata(G_OBJECT(buffer),
 					  idle_checker_id) != NULL);
+	    if (after_wait_cb != NULL)
+	      (*after_wait_cb)(callback_data);
 #if 0
 	    fprintf(stderr, "count=%d\n", count);
 #endif
@@ -1214,6 +1243,9 @@ refresh_threadlist_by_subject_txt(OchushaBulletinBoard *board,
 
   iconv_close(converter);
 
+  if (end_parsing_cb != NULL)
+    (*end_parsing_cb)(callback_data);
+
   return result;
 }
 
@@ -1233,8 +1265,15 @@ static gboolean
 ochusha_board_2ch_refresh_threadlist(OchushaBulletinBoard *board,
 				     OchushaAsyncBuffer *buffer,
 				     EachThreadCallback *each_thread_cb,
+				     StartParsingCallback *start_parsing_cb,
+				     BeforeWaitCallback *before_wait_cb,
+				     AfterWaitCallback *after_wait_cb,
+				     EndParsingCallback *end_parsing_cb,
 				     gpointer callback_data)
 {
   return refresh_threadlist_by_subject_txt(board, buffer,
-					   each_thread_cb, callback_data);
+					   each_thread_cb,
+					   start_parsing_cb, before_wait_cb,
+					   after_wait_cb, end_parsing_cb,
+					   callback_data);
 }
