@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_board_jbbs.c,v 1.11 2004/01/16 08:00:01 fuyu Exp $
+ * $Id: ochusha_board_jbbs.c,v 1.13 2004/01/21 08:41:01 fuyu Exp $
  */
 
 #include "config.h"
@@ -53,6 +53,9 @@
 #include <unistd.h>
 
 
+#define ANALYSIS_INTERVAL	200
+
+
 static void ochusha_board_jbbs_class_init(OchushaBoardJBBSClass *klass);
 static void ochusha_board_jbbs_init(OchushaBoardJBBS *board);
 
@@ -79,6 +82,10 @@ static gboolean ochusha_board_jbbs_refresh_threadlist(
 						OchushaBulletinBoard *board,
 						OchushaAsyncBuffer *buffer,
 						EachThreadCallback *cb,
+						StartParsingCallback *start_parsing_cb,
+						BeforeWaitCallback *before_wait_cb,
+						AfterWaitCallback *after_wait_cb,
+						EndParsingCallback *end_parsing_cb,
 						gpointer callback_data);
 
 static const char *ochusha_board_jbbs_get_response_character_encoding(
@@ -617,7 +624,8 @@ ochusha_board_jbbs_get_threadlist_source(OchushaBulletinBoard *board,
   board_2ch = OCHUSHA_BOARD_2CH(board);
 
   return ochusha_network_broker_read_from_url(broker, buffer, url,
-					      board_2ch->last_modified, mode);
+					      board_2ch->last_modified, mode,
+					      FALSE);
 }
 
 
@@ -685,6 +693,16 @@ undo_thread_ref(gpointer data, gpointer user_data)
 }
 
 
+static gboolean
+advance_parsing(gpointer data)
+{
+  g_object_set_qdata(G_OBJECT(data), idle_checker_id, NULL);
+  ochusha_async_buffer_broadcast((OchushaAsyncBuffer *)data);
+  OCHU_OBJECT_UNREF(data);
+  return FALSE;
+}
+
+
 /*
  * refresh_threadlist_by_subject_txt:
  *
@@ -700,11 +718,15 @@ static gboolean
 refresh_threadlist_by_subject_txt(OchushaBulletinBoard *board,
 				  OchushaAsyncBuffer *buffer,
 				  EachThreadCallback *each_thread_cb,
+				  StartParsingCallback *start_parsing_cb,
+				  BeforeWaitCallback *before_wait_cb,
+				  AfterWaitCallback *after_wait_cb,
+				  EndParsingCallback *end_parsing_cb,
 				  gpointer callback_data)
 {
   char default_buffer[PATH_MAX];
   char scan_buffer[5];
-  gboolean result;
+  gboolean result = FALSE;
   GSList *thread_list = NULL;
   GSList *old_thread_list = board->thread_list;
   iconv_t converter;
@@ -744,15 +766,21 @@ refresh_threadlist_by_subject_txt(OchushaBulletinBoard *board,
   ochusha_async_buffer_lock(buffer);
   {
     unsigned int offset = 0;
+    gboolean buffer_fixed = FALSE;
+
+    if (start_parsing_cb != NULL)
+      (*start_parsing_cb)(callback_data);
 
     while (TRUE)
       {
 	char *buffer_top = (char *)buffer->buffer;
 	char *cur_pos = buffer_top + offset;
 	unsigned int rest_of_data = buffer->length - offset;
-	char *eol_pos;
+	char *eol_pos = NULL;
+	int interval = ANALYSIS_INTERVAL;
 
 	while (rest_of_data > 0
+	       && interval-- > 0
 	       && (eol_pos = memchr(cur_pos, '\n', rest_of_data)) != NULL)
 	  {
 	    char *thread_id;
@@ -840,8 +868,7 @@ refresh_threadlist_by_subject_txt(OchushaBulletinBoard *board,
 		if (each_thread_cb != NULL
 		    && !(*each_thread_cb)(thread, callback_data))
 		  {
-		    result = FALSE;
-		    break;
+		    goto terminated;
 		  }
 	      }
 
@@ -850,30 +877,74 @@ refresh_threadlist_by_subject_txt(OchushaBulletinBoard *board,
 	    buffer_top = (char *)buffer->buffer;
 	    cur_pos = buffer_top + offset;
 	    rest_of_data = buffer->length - offset;
+	    eol_pos = NULL;
 	  }
 
-	if (buffer->fixed)
+	if (!buffer_fixed && buffer->fixed)
+	  {
+	    buffer_fixed = TRUE;
+	    eol_pos = NULL;
+	    continue;
+	  }
+
+	if (buffer->fixed && eol_pos == NULL && interval > 0)
 	  {
 	    result = TRUE;
-	    break;
+	    goto terminated;
 	  }
 
-	if (!ochusha_async_buffer_wait(buffer))
+	if (interval > 0)
 	  {
+	    size_t old_len = buffer->length;
+
+	    if (before_wait_cb != NULL)
+	      (*before_wait_cb)(callback_data);
+
+	    do
+	      {
+		if (!ochusha_async_buffer_wait(buffer))
+		  {
 #if DEBUG_ASYNC_BUFFER_MOST
-	    fprintf(stderr, "buffer has been terminated.\n");
+		    fprintf(stderr, "buffer has been terminated.\n");
 #endif
-	    result = FALSE;
-	    break;
+		    if (after_wait_cb != NULL)
+		      (*after_wait_cb)(callback_data);
+		    goto terminated;
+		  }
+	      } while (old_len >= buffer->length && !buffer->fixed);
+	    if (after_wait_cb != NULL)
+	      (*after_wait_cb)(callback_data);
 	  }
+	else
+	  {
+	    OCHU_OBJECT_REF(buffer);
+	    g_object_set_qdata(G_OBJECT(buffer), idle_checker_id, buffer);
+	    g_idle_add_full(G_PRIORITY_HIGH_IDLE + 30,
+			    advance_parsing, buffer, NULL);
+	    if (before_wait_cb != NULL)
+	      (*before_wait_cb)(callback_data);
+	    do
+	      {
+		if (!ochusha_async_buffer_wait(buffer))
+		  {
+		    if (after_wait_cb != NULL)
+		      (*after_wait_cb)(callback_data);
+		    goto terminated;
+		  }
+	      } while (g_object_get_qdata(G_OBJECT(buffer),
+					  idle_checker_id) != NULL);
+	    if (after_wait_cb != NULL)
+	      (*after_wait_cb)(callback_data);
+	  }
+	interval = ANALYSIS_INTERVAL;
 
 	if (status->state == OCHUSHA_NETWORK_BROKER_BUFFER_STATE_CACHE_IS_DIRTY)
 	  {
-	    result = FALSE;
-	    break;
+	    goto terminated;
 	  }
       }
   }
+ terminated:
   ochusha_async_buffer_unlock(buffer);
 
   if (result)
@@ -919,6 +990,9 @@ refresh_threadlist_by_subject_txt(OchushaBulletinBoard *board,
 
   iconv_close(converter);
 
+  if (end_parsing_cb != NULL)
+    (*end_parsing_cb)(callback_data);
+
   return result;
 }
 
@@ -938,9 +1012,13 @@ static gboolean
 refresh_threadlist_by_subback_html(OchushaBulletinBoard *board,
 				   OchushaAsyncBuffer *buffer,
 				   EachThreadCallback *each_thread_cb,
+				   StartParsingCallback *start_parsing_cb,
+				   BeforeWaitCallback *before_wait_cb,
+				   AfterWaitCallback *after_wait_cb,
+				   EndParsingCallback *end_parsing_cb,
 				   gpointer callback_data)
 {
-  gboolean result;
+  gboolean result = FALSE;
   GSList *thread_list = NULL;
   GSList *old_thread_list = board->thread_list;
   iconv_t converter;
@@ -993,15 +1071,21 @@ refresh_threadlist_by_subback_html(OchushaBulletinBoard *board,
   ochusha_async_buffer_lock(buffer);
   {
     unsigned int offset = 0;
+    gboolean buffer_fixed = FALSE;
+
+    if (start_parsing_cb != NULL)
+      (*start_parsing_cb)(callback_data);
 
     while (TRUE)
       {
 	char *buffer_top = (char *)buffer->buffer;
 	char *cur_pos = buffer_top + offset;
 	unsigned int rest_of_data = buffer->length - offset;
-	char *eol_pos;
+	char *eol_pos = NULL;
+	int interval = ANALYSIS_INTERVAL;
 
 	while (rest_of_data > 0
+	       && interval-- > 0
 	       && (eol_pos = memchr(cur_pos, '\n', rest_of_data)) != NULL)
 	  {
 	    char *thread_id;
@@ -1062,8 +1146,7 @@ refresh_threadlist_by_subback_html(OchushaBulletinBoard *board,
 		if (each_thread_cb != NULL
 		    && !(*each_thread_cb)(thread, callback_data))
 		  {
-		    result = FALSE;
-		    break;
+		    goto terminated;
 		  }
 	      }
 
@@ -1076,28 +1159,71 @@ refresh_threadlist_by_subback_html(OchushaBulletinBoard *board,
 	    rest_of_data = buffer->length - offset;
 	  }
 
-	if (buffer->fixed)
+	if (!buffer_fixed && buffer->fixed)
 	  {
-	    result = TRUE;
-	    break;
+	    buffer_fixed = TRUE;
+	    eol_pos = NULL;
+	    continue;
 	  }
 
-	if (!ochusha_async_buffer_wait(buffer))
+	if (buffer->fixed && eol_pos == NULL && interval > 0)
 	  {
-#if DEBUG_ASYNC_BUFFER_MOST
-	    fprintf(stderr, "buffer has been terminated.\n");
-#endif
-	    result = FALSE;
-	    break;
+	    result = TRUE;
+	    goto terminated;
 	  }
 
+	if (interval > 0)
+	  {
+	    size_t old_len = buffer->length;
+
+	    if (before_wait_cb != NULL)
+	      (*before_wait_cb)(callback_data);
+
+	    do
+	      {
+		if (!ochusha_async_buffer_wait(buffer))
+		  {
+#if DEBUG_ASYNC_BUFFER_MOST
+		    fprintf(stderr, "buffer has been terminated.\n");
+#endif
+		    if (after_wait_cb != NULL)
+		      (*after_wait_cb)(callback_data);
+		    goto terminated;
+		  }
+	      } while (old_len >= buffer->length && !buffer->fixed);
+	    if (after_wait_cb != NULL)
+	      (*after_wait_cb)(callback_data);
+	  }
+	else
+	  {
+	    OCHU_OBJECT_REF(buffer);
+	    g_object_set_qdata(G_OBJECT(buffer), idle_checker_id, buffer);
+	    g_idle_add_full(G_PRIORITY_HIGH_IDLE + 30,
+			    advance_parsing, buffer, NULL);
+	    if (before_wait_cb != NULL)
+	      (*before_wait_cb)(callback_data);
+	    do
+	      {
+		if (!ochusha_async_buffer_wait(buffer))
+		  {
+		    if (after_wait_cb != NULL)
+		      (*after_wait_cb)(callback_data);
+		    goto terminated;
+		  }
+	      } while (g_object_get_qdata(G_OBJECT(buffer),
+					  idle_checker_id) != NULL);
+	    if (after_wait_cb != NULL)
+	      (*after_wait_cb)(callback_data);
+	  }
+	interval = ANALYSIS_INTERVAL;
+
 	if (status->state == OCHUSHA_NETWORK_BROKER_BUFFER_STATE_CACHE_IS_DIRTY)
 	  {
-	    result = FALSE;
-	    break;
+	    goto terminated;
 	  }
       }
   }
+ terminated:
   ochusha_async_buffer_unlock(buffer);
 
   if (result)
@@ -1144,6 +1270,9 @@ refresh_threadlist_by_subback_html(OchushaBulletinBoard *board,
   regfree(&threadlist_entry_pattern);
   iconv_close(converter);
 
+  if (end_parsing_cb != NULL)
+    (*end_parsing_cb)(callback_data);
+
   return result;
 }
 
@@ -1163,16 +1292,27 @@ static gboolean
 ochusha_board_jbbs_refresh_threadlist(OchushaBulletinBoard *board,
 				      OchushaAsyncBuffer *buffer,
 				      EachThreadCallback *each_thread_cb,
+				      StartParsingCallback *start_parsing_cb,
+				      BeforeWaitCallback *before_wait_cb,
+				      AfterWaitCallback *after_wait_cb,
+				      EndParsingCallback *end_parsing_cb,
 				      gpointer callback_data)
 {
   switch (board->bbs_type)
     {
     case OCHUSHA_BBS_TYPE_MITINOKU:
       return refresh_threadlist_by_subback_html(board, buffer,
-						each_thread_cb, callback_data);
+						each_thread_cb,
+						start_parsing_cb,
+						before_wait_cb, after_wait_cb,
+						end_parsing_cb, callback_data);
 
     default:
       return refresh_threadlist_by_subject_txt(board, buffer,
-					       each_thread_cb, callback_data);
+					       each_thread_cb,
+					       start_parsing_cb,
+					       before_wait_cb, after_wait_cb,
+					       end_parsing_cb, callback_data);
+
     }
 }
