@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_network_broker.c,v 1.49 2004/01/21 08:41:01 fuyu Exp $
+ * $Id: ochusha_network_broker.c,v 1.50 2004/01/22 07:48:32 fuyu Exp $
  */
 
 #include "config.h"
@@ -63,6 +63,7 @@
 #define LOG_MESSAGE_SIZE	4096
 
 #define POLLING_INTERVAL_MILLIS	100
+#define POLLING_TIMEOUT_COUNT	300
 
 
 static void ochusha_network_broker_class_init(OchushaNetworkBrokerClass *klass);
@@ -407,6 +408,7 @@ struct _WorkerSyncObject
   pthread_cond_t cond;
   guint timeout_id;
   guint poll_id;
+  volatile guint timeout_count;
   volatile gboolean read_ok;
   volatile gboolean finished;
   volatile gboolean interrupted;
@@ -553,6 +555,7 @@ timeout_cb(WorkerSyncObject *sync_object)
   g_return_val_if_fail(OCHUSHA_IS_WORKER_SYNC_OBJECT(sync_object), FALSE);
 
   LOCK_WORKER_SYNC_OBJECT(sync_object);
+  sync_object->timeout_count++;
   finished = sync_object->finished;
   if (pthread_cond_signal(&sync_object->cond) != 0)
     {
@@ -694,7 +697,7 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
 
   if (!setup_common_request_headers(broker, request, FALSE))
     {
-      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
 			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_INVALID_PROXY,
 			_("Proxy setting may be wrong."));
 
@@ -708,7 +711,7 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
       return FALSE;
     }    
 
-  g_signal_emit_by_name(G_OBJECT(buffer), "access_started", 0);
+  g_signal_emit_by_name(G_OBJECT(buffer), "access_started");
 
   snprintf(message, LOG_MESSAGE_SIZE, _("Starting GET request: %s\n"), url);
   ochusha_network_broker_output_log(broker, message);
@@ -744,7 +747,27 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
 	  current_status = ghttp_get_status(request);
 	}
       else
-	UNLOCK_WORKER_SYNC_OBJECT(sync_object);
+	{
+	  if (sync_object->read_ok)
+	    sync_object->timeout_count = 0;
+	  else if (sync_object->timeout_count > POLLING_TIMEOUT_COUNT)
+	    {
+	      sync_object->finished = TRUE;
+	      UNLOCK_WORKER_SYNC_OBJECT(sync_object);
+
+	      ghttp_request_destroy(request);
+	      args->request = NULL;
+
+	      snprintf(message, LOG_MESSAGE_SIZE,
+		       _("Access Timeout: %s\n"), url);
+	      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
+			    OCHUSHA_NETWORK_BROKER_FAILURE_REASON_TIMEOUT,
+			    message);
+	      ochusha_network_broker_output_log(broker, message);
+	      return FALSE;
+	    }
+	  UNLOCK_WORKER_SYNC_OBJECT(sync_object);
+	}
 
 #if DEBUG_NETWORK_MOST
       if (state == ghttp_error)
@@ -838,7 +861,7 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
       if (body_len > 0)
 	{
 	  if (current_status.bytes_total > 0)
-	    g_signal_emit_by_name(G_OBJECT(buffer), "access_progressed", 0,
+	    g_signal_emit_by_name(G_OBJECT(buffer), "access_progressed",
 				  current_status.bytes_read,
 				  current_status.bytes_total);
 
@@ -1004,7 +1027,7 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
       else
 	status->date = NULL;
 
-      g_signal_emit_by_name(G_OBJECT(buffer), "access_finished", 0);
+      g_signal_emit_by_name(G_OBJECT(buffer), "access_finished");
 
       ghttp_request_destroy(request);
       args->request = NULL;
@@ -1031,7 +1054,7 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
       snprintf(message, LOG_MESSAGE_SIZE,
 	       _("Access failed(%s): status_code=%d (%s)\n"),
 	       url, status_code, error_message);
-      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
 			    OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
 			    message);
 
@@ -1062,7 +1085,7 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
     {
       snprintf(message, LOG_MESSAGE_SIZE, _("Access failed(%s): %d (%s)\n"),
 	       url, status_code, ghttp_reason_phrase(request));
-      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
 			    OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
 			    message);
       ochusha_network_broker_output_log(broker, message);
@@ -1251,7 +1274,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	url = NULL;
 
       status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
-      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
 		OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
 		_("Access terminated."));
 
@@ -1370,7 +1393,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	{
 	  /* ۤäƥåϤɤΤ */
 	  status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_CACHE_IS_OK;
-	  g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+	  g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
 			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_INVALID_PROXY,
 			_("Proxy setting may be wrong."));
 
@@ -1380,7 +1403,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	  goto finish_try_update_cache;
 	}
 
-      g_signal_emit_by_name(G_OBJECT(buffer), "access_started", 0);
+      g_signal_emit_by_name(G_OBJECT(buffer), "access_started");
 
       snprintf(message, LOG_MESSAGE_SIZE, _("Updating cache file: %s\n"),
 	       url);
@@ -1400,7 +1423,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 
 	  snprintf(message, LOG_MESSAGE_SIZE,
 		   _("Access terminated: %s\n"), url);
-	  g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+	  g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
 		OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
 		message);
 	  ochusha_network_broker_output_log(broker, message);
@@ -1459,8 +1482,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 		  status->state
 		    = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_CACHE_IS_OK;
 
-		  g_signal_emit_by_name(G_OBJECT(buffer),
-					"access_finished", 0);
+		  g_signal_emit_by_name(G_OBJECT(buffer), "access_finished");
 
 		  ghttp_request_destroy(request);
 		  args->request = NULL;
@@ -1546,7 +1568,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	  if (body_len > 0)
 	    {
 	      if (current_status.bytes_total > 0)
-		g_signal_emit_by_name(G_OBJECT(buffer), "access_progressed", 0,
+		g_signal_emit_by_name(G_OBJECT(buffer), "access_progressed",
 				      current_status.bytes_read,
 				      current_status.bytes_total);
 
@@ -1582,7 +1604,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 
 		  snprintf(message, LOG_MESSAGE_SIZE,
 			   _("Access terminated: %s\n"), url);
-		  g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+		  g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
 			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
 			message);
 		  ochusha_network_broker_output_log(broker, message);
@@ -1621,7 +1643,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 		  snprintf(message, LOG_MESSAGE_SIZE,
 			   _("Access terminated: %s\n"),
 			   url);
-		  g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+		  g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
 			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
 			message);
 		  ochusha_network_broker_output_log(broker, message);
@@ -1667,7 +1689,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	status->date = NULL;
 
       status->status_code = status_code;
-      g_signal_emit_by_name(G_OBJECT(buffer), "access_finished", 0);
+      g_signal_emit_by_name(G_OBJECT(buffer), "access_finished");
 
       ghttp_request_destroy(request);
       args->request = NULL;
@@ -1696,7 +1718,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 
 	  snprintf(message, LOG_MESSAGE_SIZE,
 		   _("Access terminated: %s\n"), url);
-	  g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+	  g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
 		OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
 		message);
 	  ochusha_network_broker_output_log(broker, message);
@@ -1730,7 +1752,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
     {
       snprintf(message, LOG_MESSAGE_SIZE,
 	       _("Access failed(%s): %s\n"), url, error_message);
-      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
 			    OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
 			    message);
       ochusha_network_broker_output_log(broker, message);
@@ -1743,7 +1765,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	  snprintf(message, LOG_MESSAGE_SIZE,
 		   _("Access failed(%s): %d (%s)\n"),
 		   url, status_code, ghttp_reason_phrase(request));
-	  g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+	  g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
 				OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
 				message);
 	  ochusha_network_broker_output_log(broker, message);
@@ -1753,7 +1775,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	  snprintf(message, LOG_MESSAGE_SIZE,
 		   _("Access failed(%s): unknown reason.\n"),
 		   url);
-	  g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+	  g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
 				OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
 				message);
 	  ochusha_network_broker_output_log(broker, message);
@@ -1810,7 +1832,7 @@ refresh_cache_after_read(WorkerThread *employee, OchushaAsyncBuffer *buffer)
   if (!ochusha_async_buffer_active_ref(buffer))
     {
       status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
-      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
 		OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
 		_("Access terminated."));
       goto finish_refresh_cache_after_read;
@@ -1820,7 +1842,7 @@ refresh_cache_after_read(WorkerThread *employee, OchushaAsyncBuffer *buffer)
   if (args == NULL)
     {
       status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
-      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
 		OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
 		_("Access terminated."));
       goto finish_refresh_cache_after_read;
@@ -1847,14 +1869,14 @@ refresh_cache_after_read(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	    {	/* å夬ä */
 	      if (args->if_modified_since != NULL)
 		status->last_modified = G_STRDUP(args->if_modified_since);
-	      g_signal_emit_by_name(G_OBJECT(buffer), "access_finished", 0);
+	      g_signal_emit_by_name(G_OBJECT(buffer), "access_finished");
 	    }
 	  else
 	    {	/* ̿˼ԤΤǻʤåȤ */
 	      snprintf(message, LOG_MESSAGE_SIZE,
 		       _("Use cached file(%s): due to networking error.\n"),
 		       args->url);
-	      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+	      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
 				OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
 				message);
 	      ochusha_network_broker_output_log(broker, message);
@@ -1868,7 +1890,7 @@ refresh_cache_after_read(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	      snprintf(message, LOG_MESSAGE_SIZE,
 		       _("Couldn't find cache file: %s\n"),
 		       args->url);
-	      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+	      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
 				OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
 				message);
 	      ochusha_network_broker_output_log(broker, message);
@@ -1876,11 +1898,11 @@ refresh_cache_after_read(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	  else
 	    {
 	      if (buffer->state != OCHUSHA_ASYNC_BUFFER_TERMINATED)
-		g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+		g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
 				OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
 				_("Couldn't read data via the network."));
 	      else
-		g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+		g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
 			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
 			_("Access terminated."));
 
@@ -1976,7 +1998,7 @@ background_read_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 
   if (ochusha_async_buffer_read_file(buffer, fd))
     {
-      g_signal_emit_by_name(G_OBJECT(buffer), "access_finished", 0);
+      g_signal_emit_by_name(G_OBJECT(buffer), "access_finished");
     }
   else
     {
@@ -2036,7 +2058,7 @@ ochusha_network_broker_read_from_url(OchushaNetworkBroker *broker,
   if (mode == OCHUSHA_NETWORK_BROKER_CACHE_IGNORE && broker->config->offline)
     {
 #if 0	/* ƤӽФ¦Թǰ褺ȥ */
-      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
 			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_NOT_CONNECTED,
 			_("Network not connected on offline mode."));
 #endif
@@ -2088,7 +2110,7 @@ ochusha_network_broker_read_from_url(OchushaNetworkBroker *broker,
 	{
 	  /* åʤĥå夷Ȥʤ */
 #if 0	/* ƤӽФ¦Թǰ褺ȥ */
-	  g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+	  g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
 			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_CACHE_NOT_FOUND,
 			_("Couldn't find cache that should exist."));
 #endif
@@ -2098,7 +2120,7 @@ ochusha_network_broker_read_from_url(OchushaNetworkBroker *broker,
       if (broker->config->offline)
 	{
 #if 0	/* ƤӽФ¦Թǰ褺ȥ */
-	  g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+	  g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
 			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_NOT_CONNECTED,
 			_("Network not connected on offline mode."));
 #endif
@@ -2166,7 +2188,7 @@ http_read_from_url_synchronously(OchushaNetworkBroker *broker,
 
   if (!setup_common_request_headers(broker, request, FALSE))
     {
-      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
 			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_INVALID_PROXY,
 			_("Proxy setting may be wrong."));
 
@@ -2175,7 +2197,7 @@ http_read_from_url_synchronously(OchushaNetworkBroker *broker,
       return FALSE;
     }    
 
-  g_signal_emit_by_name(G_OBJECT(buffer), "access_started", 0);
+  g_signal_emit_by_name(G_OBJECT(buffer), "access_started");
 
   ghttp_set_sync(request, ghttp_sync);
   ghttp_prepare(request);
@@ -2219,7 +2241,7 @@ http_read_from_url_synchronously(OchushaNetworkBroker *broker,
     {
       ghttp_current_status current_status = ghttp_get_status(request);
       if (current_status.bytes_total > 0)
-	g_signal_emit_by_name(G_OBJECT(buffer), "access_progressed", 0,
+	g_signal_emit_by_name(G_OBJECT(buffer), "access_progressed",
 			      current_status.bytes_read,
 			      current_status.bytes_total);
 
@@ -2298,7 +2320,7 @@ http_read_from_url_synchronously(OchushaNetworkBroker *broker,
 #if DEBUG_NETWORK_MOST
   fprintf(stderr, "http_read_from_url_synchronously succeeded.\n");
 #endif
-  g_signal_emit_by_name(G_OBJECT(buffer), "access_finished", 0);
+  g_signal_emit_by_name(G_OBJECT(buffer), "access_finished");
 
   return TRUE;
 
@@ -2308,7 +2330,7 @@ http_read_from_url_synchronously(OchushaNetworkBroker *broker,
 #endif
   if (error_message != NULL)
     {
-      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
 			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
 			error_message);
 #if 0
@@ -2321,7 +2343,7 @@ http_read_from_url_synchronously(OchushaNetworkBroker *broker,
       char message[LOG_MESSAGE_SIZE];
       snprintf(message, LOG_MESSAGE_SIZE, "Access failed: %d (%s)\n",
 	       status_code, ghttp_reason_phrase(request));
-      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
 			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
 			message);
 #if 0
@@ -2329,7 +2351,7 @@ http_read_from_url_synchronously(OchushaNetworkBroker *broker,
 #endif
     }
   else
-    g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+    g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
 			  OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
 			  _("Access failed: unknown reason."));
 
