@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_thread_jbbs.c,v 1.32 2004/01/20 13:52:37 fuyu Exp $
+ * $Id: ochusha_thread_jbbs.c,v 1.33 2004/01/22 07:48:32 fuyu Exp $
  */
 
 #include "config.h"
@@ -1170,7 +1170,7 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
 
   if (!ochusha_async_buffer_active_ref(html_buffer))
     {
-      g_signal_emit_by_name(G_OBJECT(dat_buffer), "access_failed", 0,
+      g_signal_emit_by_name(G_OBJECT(dat_buffer), "access_failed",
 		OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
 		_("Access terminated."));
       goto terminated;
@@ -1178,7 +1178,7 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
 
   if (!ochusha_async_buffer_active_ref(dat_buffer))
     {
-      g_signal_emit_by_name(G_OBJECT(dat_buffer), "access_failed", 0,
+      g_signal_emit_by_name(G_OBJECT(dat_buffer), "access_failed",
 		OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
 		_("Access terminated."));
       ochusha_async_buffer_active_unref(html_buffer);
@@ -1488,7 +1488,7 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
 #if DEBUG_HTML_TO_DAT_CONVERTER_MOST
 	  fprintf(stderr, "conversion done.\n");
 #endif
-	  g_signal_emit_by_name(G_OBJECT(dat_buffer), "access_finished", 0);
+	  g_signal_emit_by_name(G_OBJECT(dat_buffer), "access_finished");
 
 	  if (previous_line != NULL)
 	    {
@@ -1503,7 +1503,7 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
 #if DEBUG_HTML_TO_DAT_CONVERTER_MOST
 	  fprintf(stderr, "wait terminated.\n");
 #endif
-	  g_signal_emit_by_name(G_OBJECT(dat_buffer), "access_failed", 0,
+	  g_signal_emit_by_name(G_OBJECT(dat_buffer), "access_failed",
 		OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
 		_("Access terminated."));
 	  if (previous_line != NULL)
@@ -1714,7 +1714,7 @@ ochusha_thread_jbbs_fake_responses_source(OchushaThread2ch *thread_2ch,
     {
       if (dat_buffer != NULL)
 	{
-	  g_signal_emit_by_name(G_OBJECT(dat_buffer), "access_failed", 0,
+	  g_signal_emit_by_name(G_OBJECT(dat_buffer), "access_failed",
 			  OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
 			  _("Access failed: unknown reason."));
 	  ochusha_async_buffer_fix(dat_buffer);
